#!/bin/bash
echo $*
#szVersion="@(#)sep_oracle.sh	1.2 00/01/24 16:36:54"
#szVersion="$Id @(#)sep_oracle.sh	1.2 00/01/24 16:36:54$"
###########################################################################
# sm_oracle_client
#
# Description: 
# SEP Oracle 8 Backup utility for client based Oracle Backup and Restore 
# This utility builds and runs RMAN scripts 
#
# Syntax:
szUsage="$0 -b|r <-l level> -T <TargetDBS> -R <RCVCATConnectString> -S <SesamServer> -j <SesamJob> <TableSpace>"
# Arguments:
# -l: Backup operation. By default, this is a full, on-line, single channel backup
# -r: Restore operation. By default, this is a full on-line, single channel restore.
# -o <delete|recover>: During  backup archive logs, delete them after backup. For restore, do database recovery after restore
# -T <Target Database>: Oracle SID for target database
# -R <Recovery Catalog Connect String>
# -S <SesamServer>: Sesam Server host name
# -j <SesamJob>: Sesam Job name, under which backup is running
# -v: debug on (set -x)
# -m <pool name>: Sesam media pool to use for backup 
# -p <number> defines the number of backup channels
# <TableSpace> defines a single tablespace to backup or restore. If not given, backup/restore whole database
# Return value: 
# 0 for OK, 1 for error
#
# Messaging: 
# Forwarded from RMAN
###########################################################################


if [ "$USER" = "root" ]
then
su - oracle -c "$0 $*"
exit
fi



echo $*

echo "$(date): `hostname`: `basename $0`-001: Info: Starting `basename $0`"


# Configuration


# RMAN command
szRmanCMD="rman"

# Destination directory for DISK backup
#szDiskDestination="$ORACLE_HOME/sob/backup"

szOperation=""

if [[ $1 = "" ]]; 
then
	echo "$(date): `hostname`: `basename $0`-101: Error: No or wrong parameters. Exiting."
	echo "$(date): `hostname`: `basename $0`-050: Info:  $szUsage"
	exit 1
fi

if [[ "$ORACLE_HOME" = "" ]]
then
	echo "$(date): `hostname`: `basename $0`-102: Error: ORACLE_HOME environment variable not defined. Exiting."
	exit 1
fi

#logging call of sm_oracle_client
szRmanLog=$ORACLE_HOME/sob/sm_oracle_client.log
echo $(date): Starting `basename $0` $* >>$szRmanLog

# Getting options
sz_opts=":bracvS:j:t:o:d:p:i:m:T:l:R:"
while getopts $sz_opts c_opt; do
	case $c_opt in
        x)      szOperation=Execute;;
	b)	szOperation=Backup;;	
	r)	szOperation=Restore;;
	l)	szLevel=$OPTARG;; 
	S)	szSesamServer=$OPTARG;;
	j)	szSesamJob=$OPTARG;;
	T)	szTarget=$OPTARG;;
	R)	szRCVCAT=$OPTARG;;
	m)	szMediaPool=$OPTARG;;
	i)	szDriveInterface=$OPTARG;;
	v)	set -x;;
	p)	iChannelMax=$OPTARG;;
	d)	szDrive=$OPTARG;;
        o)      szOption=$OPTARG;;
	\?)	echo "$szUsage"
		exit 1;;
	esac
done
shift `expr $OPTIND - 1`
szTablespace=$*

#TBD check for empty arguments
#argument parsing done
#
if [[ $szOperation = "" ]]
then 
  echo $szUsage
  exit 1
fi

# Export sesam variables, if any
if [[ $szDrive != "" ]]
then
	export szDriveNum=$szDrive
fi

if [[ $szMediaPool != "" ]]
then
	export szMediaPool=$szMediaPool
fi

if [[ $szDriveInterface != "" ]]
then
	export szDriveInterface=$szDriveInterface
fi

if [[  $szTablespace = "" ]]
then 
  szBackupSrc=database
else
  szBackupSrc="tablespace \"$szTablespace\""
fi

if [[  $szDrive = "DISK" ]]
then
  szBckTarget="type disk;"
  # Destination directory for DISK backup. / at the and is important
  szDiskDestination="$ORACLE_HOME/sob/backup/"
else
  szBckTarget=" type 'SBT_TAPE' parms 'ENV=(SESAM_SERVER=$szSesamServer,SESAM_JOB=$szSesamJob,SESAM_DRIVE=$szDriveNum,SESAM_POOL=$szMediaPool,SOB_TRACE=3,SOB_LOGFILE=$ORACLE_HOME/sob/sob1.log)';" 
fi

if [[  $szOption = "delete" ]]
then
  szDeleteInput="delete input"
fi


if [[  $szOption = "recover" ]]
then
  szRecoverDatabase="recover database;"
fi

if [[  $szOption = "delete" ]]
then
  szDeleteInput="delete input"
fi



szLevel=${szLevel:-copy}

if [ `expr $szLevel : '.*arch.*'` -eq 0 -a  "$szDeleteInput" = "delete input" ]
  then
   echo $szUsage
   exit 1 
fi


if [[ $szOperation = "Backup" ]]
then 
szLevel=${szLevel:-copy}
case $szLevel in 
    copy*) szBackupCmd="backup full"
           szFormatBackup="${szDiskDestination}Oc%d%u" 
           ;;
    full*) szBackupCmd="backup incremental level 0"
           szFormatBackup="${szDiskDestination}Of%d%u"
           ;;
    diff*) szBackupCmd="backup incremental level 1 cumulative"
           szFormatBackup="${szDiskDestination}Od%d%u" 
           ;;
    incr*) szBackupCmd="backup incremental level 1"
           szFormatBackup="${szDiskDestination}Oi%d%u" 
           ;;
    arch*) szBackupSrc="archivelog all $szDeleteInput"
           szBackupCmd="backup"
           szFormatBackup="${szDiskDestination}Oa%d%u"
           ;;
esac
fi

if [[ $szOperation = "Restore" ]]
then 
  iChannelMax=1
  szBackupCmd="restore"
  szLevel="rest"
fi

# Allocate multiple channels

iChannelMax=${iChannelMax:-1}

count=1
while [ $count -le $iChannelMax ]
  do
    channels=$channels"allocate channel ${ORACLE_SID}$count $szBckTarget"
    re_channels=$re_channels"release channel ${ORACLE_SID}$count;"
    count=`expr $count + 1`
done


if [[  $szOption = "recover" ]]
then
  szRecoverDatabase="recover $szBackupSrc;"
fi

if [[  $szOption = "delete" ]]
then
  szDeleteInput="delete input"
fi


echo "$(date): `hostname`: `basename $0`-011: Info:  Sesam Server  -> $szSesamServer"
echo "$(date): `hostname`: `basename $0`-012: Info:  Sesam Job     -> $szSesamJob"
echo "$(date): `hostname`: `basename $0`-013: Info:  Oracle SID    -> $szTarget"
echo "$(date): `hostname`: `basename $0`-014: Info:  Backup source -> $szBackupSrc"
echo "$(date): `hostname`: `basename $0`-015: Info:  Backup level  -> $szLevel ----> $szBackupCmd"
echo "$(date): `hostname`: `basename $0`-016: Info:  Backup channels  -> $iChannelMax"

# Run Operation
iStatus=1
echo "$(date): `hostname`: `basename $0`-020: Info:  Building and running RMAN script $szLevel"


case $szLevel in
        arch) 
	# 	
	export ORACLE_SID=$szTarget
        echo "	connect target 
	run { 
        sql 'alter system archive log current';
	`echo $channels` 
	`echo $szBackupCmd $szBackupSrc` 
	format '$szFormatBackup'; 
	`echo $re_channels` 
	}
	" | tee -a $szRmanLog | $szRmanCMD rcvcat $szRCVCAT
	iStatus=$?
        ;;

        copy|full|diff|incr) 
	# 	
	export ORACLE_SID=$szTarget
	echo "connect target 
	run { $channels 
        $szBackupCmd $szBackupSrc 
        format '$szFormatBackup'; 
        $re_channels }
	" | tee -a $szRmanLog | $szRmanCMD rcvcat $szRCVCAT
	iStatus=$?
        ;;

        copy*arch|full*arch|diff*arch|incr*arch)
        #
        export ORACLE_SID=$szTarget
        echo "connect target
        run {
        `echo $channels`
        `echo $szBackupCmd $szBackupSrc`
        format '$szFormatBackup';
        sql 'alter system archive log current';
        backup
        format '$szFormatBackup'
        (archivelog all $szDeleteInput);
        `echo $re_channels`
        }
	" | tee -a $szRmanLog | $szRmanCMD rcvcat $szRCVCAT
	iStatus=$?
	;;

	rest*)
	export ORACLE_SID=$szTarget
	echo "connect target
	run {
	`echo $channels`
	`echo $szBackupCmd $szBackupSrc`;
	`echo $szRecoverDatabase`
	`echo $re_channels`
	}
	" | tee -a $szRmanLog | $szRmanCMD rcvcat $szRCVCAT
	iStatus=$?
	;;
	*)			echo "$szUsage";;
esac

if [[  $iStatus = "0" ]]
  then
    echo "$(date): `hostname`: `basename $0`-002: Info:  RMAN returned sucessfully. Status: $iStatus "
  else
    echo "$(date): `hostname`: `basename $0`-102: Error: RMAN returned with error. Status: $iStatus "
  fi


exit $iStatus

